//============================================================+
// File name   : calkXY.jsx
// Version     : 1.0.000
// Begin       : 2021-09-03
// Last Update : 2021-09-03
// Authors     : t.kenro - https://hoshifuru.jp - pr@hoshifuru.jp
// -------------------------------------------------------------------
// Copyright (C) 2021 t.kenro - hoshifuru.jp
//
// v1.0.0 20210903 calkXY.js v1.0.700をそのまま利用。

//--------------------
//Calculate X Y
//--------------------

//--------------------
//各座標からAI上の座標を取得
//--------------------
function calkXYfromXXX(d1,d2,dataType,Eangle){
	if(typeof Eangle === "undefined") Eangle = 23.439291;
	var XY = {};
	if(dataType == "Equatorial"){
		XY = calkXYfromEquatorial(d1,d2,Eangle);
	}else if(dataType == "Horizontal"){
		XY = calkXYfromHorizontal(d1,d2,Eangle);
	}else if(dataType == "Ecliptic"){
		XY= calkXYfromEcliptic(d1,d2,Eangle);
	}else if(dataType == "Galactic"){
		XY= calkXYfromGalactic(d1,d2,Eangle);
	}else{
		XY = calkXYfromEquatorial(d1,d2,Eangle);
	}
	return XY;
}

//赤道座標 => AI上の座標
function calkXYfromEquatorial(RAdeg,DECdeg,Eangle){
	if(typeof Eangle === "undefined") Eangle = 23.439291;

	var XYZ = putOnSphere(RAdeg,DECdeg);

	switch(cfg.chart.to){
		case "Equatorial":
			break;
		case "Horizontal":
			XYZ = convert_Equatorial_to_Horizontal(XYZ);
			XYZ = correctH(XYZ);
			break;
		case "Galactic":
			XYZ = convert_Equatorial_to_Galactic(XYZ); break;
		case "Ecliptic":
			XYZ = convert_Equatorial_to_Ecliptic(XYZ,Eangle); break;
		default:
			break;
	}

	var XY = culkXYfromXYZ(XYZ,cfg.chart.projection);

	return XY;
}
//地平座標 => AI上の座標
function calkXYfromHorizontal(Adeg,Hdeg,Eangle){
	if(typeof Eangle === "undefined") Eangle = 23.439291;

	var XYZ = putOnSphere(180-Adeg,Hdeg);

	switch(cfg.chart.to){
		case "Equatorial":
			XYZ = convert_Horizontal_to_Equatorial(XYZ); break;
		case "Horizontal":
			break;
		case "Galactic":
			XYZ = convert_Horizontal_to_Galactic(XYZ); break;
		case "Ecliptic":
			XYZ = convert_Horizontal_to_Ecliptic(XYZ,Eangle); break;
		default:
			XYZ = convert_Horizontal_to_Equatorial(XYZ); break;
	}

	XY = culkXYfromXYZ(XYZ,cfg.chart.projection);
	return XY;
}
//黄道座標 => AI上の座標
function calkXYfromEcliptic(Ldeg,Bdeg,Eangle,PIdeg){
	if(typeof Eangle === "undefined") Eangle = 23.439291;
	if(typeof PIdeg === "undefined") PIdeg = 0;

	var XYZ = putOnSphere(Ldeg,Bdeg);

	switch(cfg.chart.to){
		case "Equatorial":
			XYZ = convert_Ecliptic_to_Equatorial(XYZ,Eangle); break;
		case "Horizontal":
			XYZ = convert_Ecliptic_to_Horizontal(XYZ,Eangle);
			XYZ = correctH(XYZ,PIdeg);
			break;
		case "Galactic":
			XYZ = convert_Ecliptic_to_Galactic(XYZ,Eangle); break;
		case "Ecliptic":
			break;
		default:
			XYZ = convert_Ecliptic_to_Equatorial(XYZ,Eangle); break;
	}

	XY = culkXYfromXYZ(XYZ,cfg.chart.projection);
	return XY;
}
//銀河座標 => AI上の座標
function calkXYfromGalactic(ldeg,bdeg,Eangle){
	if(typeof Eangle === "undefined") Eangle = 23.439291;

	var XYZ = putOnSphere(ldeg,bdeg);

	switch(cfg.chart.to){
		case "Equatorial":
			XYZ = convert_Galactic_to_Equatorial(XYZ); break;
		case "Horizontal":
			XYZ = convert_Galactic_to_Horizontal(XYZ);
			XYZ = correctH(XYZ);
			break;
		case "Galactic":
			break;
		case "Ecliptic":
			XYZ = convert_Galactic_to_Ecliptic(XYZ,Eangle); break;
		default:
			XYZ = convert_Galactic_to_Equatorial(XYZ); break;
	}

	XY = culkXYfromXYZ(XYZ,cfg.chart.projection);
	return XY;
}

function putOnSphere(Adeg,Hdeg){
	var XYZ		= {	X : 0,	Y : 0,	Z : 0	};
	XYZ.X = Math.cos(DegToRad(Hdeg)) * Math.cos(DegToRad(Adeg));
	XYZ.Y = -Math.cos(DegToRad(Hdeg)) * Math.sin(DegToRad(Adeg));
	XYZ.Z = Math.sin(DegToRad(Hdeg));
	return XYZ;
}

function correctH(XYZ,PIdeg){
	if(typeof PIdeg === "undefined") PIdeg = 0;

	if(!cfg.isCorrectH){
		return XYZ;
	}

	var	H = RadToDeg(	Math.asin(XYZ.Z)	);
	var	A = Math.atan2(XYZ.Y,XYZ.X);

	var cHbyParallax	= PIdeg;
	var cHbyAtmosphere	= 0.0167/Math.tan(DegToRad(	H+8.6/(H+4.4)	));
	var cH				= DegToRad(	H - cHbyParallax + cHbyAtmosphere	);

	var rXYZ = {};
	rXYZ.X = Math.cos(cH) * Math.cos(A);
	rXYZ.Y = Math.cos(cH) * Math.sin(A);
	rXYZ.Z = Math.sin(cH);

	return rXYZ;
}

//----------
// 座標変換
// 参考文献：赤道座標から銀河座標への変換 http://www.isas.jaxa.jp/home/ebisawalab/ebisawa/TEACHING/2010Komaba/2010Komaba/node19.html
// 参考文献：「日の出日の入りの計算」　長沢工　地人書館
//----------
//赤道座標 => 地平座標
function convert_Equatorial_to_Horizontal(XYZ){
	var rXYZ = XYZ;
	rXYZ = rotateZ(rXYZ.X,rXYZ.Y,rXYZ.Z,getLSThr()/24*360);
	rXYZ = rotateY(rXYZ.X,rXYZ.Y,rXYZ.Z,-(90-cfg.Location.Lat));
	return rXYZ;
}
//赤道座標 => 銀河座標
function convert_Equatorial_to_Galactic(XYZ){
	var rXYZ = XYZ;
	rXYZ = rotateZ(rXYZ.X,rXYZ.Y,rXYZ.Z,266.4051);
	rXYZ = rotateY(rXYZ.X,rXYZ.Y,rXYZ.Z,-28.936175);
	rXYZ = rotateX(rXYZ.X,rXYZ.Y,rXYZ.Z,58.59866);
	return rXYZ;
}
//赤道座標 => 黄道座標
function convert_Equatorial_to_Ecliptic(XYZ,Eangle){
	if(typeof Eangle === "undefined") Eangle = 23.439291;
	var rXYZ = XYZ;
	rXYZ = rotateX(rXYZ.X,rXYZ.Y,rXYZ.Z,Eangle);
	return rXYZ;
}
//地平座標 => 赤道座標
function convert_Horizontal_to_Equatorial(XYZ){
	var rXYZ = XYZ;
	rXYZ = rotateY(rXYZ.X,rXYZ.Y,rXYZ.Z,90-cfg.Location.Lat);
	rXYZ = rotateZ(rXYZ.X,rXYZ.Y,rXYZ.Z,-getLSThr()/24*360);
	return rXYZ;
}
//地平座標 => 銀河座標
function convert_Horizontal_to_Galactic(XYZ){
	var rXYZ = XYZ;
	rXYZ = convert_Horizontal_to_Equatorial(rXYZ);
	rXYZ = convert_Equatorial_to_Galactic(rXYZ);
	return rXYZ;
}
//地平座標 => 黄道座標
function convert_Horizontal_to_Ecliptic(XYZ,Eangle){
	if(typeof Eangle === "undefined") Eangle = 23.439291;
	var rXYZ = XYZ;
	rXYZ = convert_Horizontal_to_Equatorial(rXYZ);
	rXYZ = convert_Equatorial_to_Ecliptic(rXYZ,Eangle);
	return rXYZ;
}
//銀河座標 => 赤道座標
function convert_Galactic_to_Equatorial(XYZ){
	var rXYZ = XYZ;
	rXYZ = rotateX(rXYZ.X,rXYZ.Y,rXYZ.Z,-58.59866);
	rXYZ = rotateY(rXYZ.X,rXYZ.Y,rXYZ.Z,28.936175);
	rXYZ = rotateZ(rXYZ.X,rXYZ.Y,rXYZ.Z,-266.4051);
	return rXYZ;
}
//銀河座標 => 地平座標
function convert_Galactic_to_Horizontal(XYZ){
	var rXYZ = XYZ;
	rXYZ = convert_Galactic_to_Equatorial(rXYZ);
	rXYZ = convert_Equatorial_to_Horizontal(rXYZ);
	return rXYZ;
}
//銀河座標 => 黄道座標
function convert_Galactic_to_Ecliptic(XYZ,Eangle){
	if(typeof Eangle === "undefined") Eangle = 23.439291;
	var rXYZ = XYZ;
	rXYZ = convert_Galactic_to_Equatorial(rXYZ);
	rXYZ = convert_Equatorial_to_Ecliptic(rXYZ,Eangle);
	return rXYZ;
}
//黄道座標 => 座標
function convert_Ecliptic_to_Equatorial(XYZ,Eangle){
	if(typeof Eangle === "undefined") Eangle = 23.439291;
	var rXYZ = XYZ;
	rXYZ = rotateX(rXYZ.X,rXYZ.Y,rXYZ.Z,-Eangle);
	return rXYZ;
}
//黄道座標 => 地平座標
function convert_Ecliptic_to_Horizontal(XYZ,Eangle){
	if(typeof Eangle === "undefined") Eangle = 23.439291;
	var rXYZ = XYZ;
	rXYZ = convert_Ecliptic_to_Equatorial(XYZ,Eangle);
	rXYZ = convert_Equatorial_to_Horizontal(rXYZ);
	return rXYZ;
}
//黄道座標 => 銀河座標
function convert_Ecliptic_to_Galactic(XYZ,Eangle){
	if(typeof Eangle === "undefined") Eangle = 23.439291;
	var rXYZ = XYZ;
	rXYZ = convert_Ecliptic_to_Equatorial(XYZ,Eangle);
	rXYZ = convert_Equatorial_to_Galactic(rXYZ);
	return rXYZ;
}

//-------
// 球面上の座標からAI上の座標を取得
//-------

var addR = {
	Z: 180,	// 視線方向の経度
	Y: 0,	// 視線方向の緯度
	X: 0	// 視線の回転
};

//指定した図法の計算式からPDF上の座標を取得して返す
function culkXYfromXYZ(XYZ,projection){

	//方向・高さ・回転に対応。
	if(addR.Z != 0){	XYZ = rotateZ(XYZ.X,XYZ.Y,XYZ.Z,addR.Z);	}
	if(addR.Y != 0){	XYZ = rotateY(XYZ.X,XYZ.Y,XYZ.Z,addR.Y);	}
	if(addR.X != 0){	XYZ = rotateX(XYZ.X,XYZ.Y,XYZ.Z,addR.X);	}

	var r = Math.sqrt(XYZ.X*XYZ.X + XYZ.Y*XYZ.Y + XYZ.Z*XYZ.Z);	
	if(r>1){
		XYZ.X = XYZ.X / r;
		XYZ.Y = XYZ.Y / r;
		XYZ.Z = XYZ.Z / r;
	}

	var XY = {};
	switch(projection){
		case "AzimuthalEquidistantProjection":
			XY = AzimuthalEquidistantProjection(XYZ); break;
		case "StereographicProjection":
			XY = StereographicProjection(XYZ); break;
		case "EquirectangularProjection":
			XY = EquirectangularProjection(XYZ); break;
		case "HammerProjection":
			XY = HammerProjection(XYZ); break;
		case "MollweideProjection":
			XY = MollweideProjection(XYZ); break;
		case "MercatorProjection":
			XY = MercatorProjection(XYZ); break;
		case "LambertAzimuthalEqualAreaProjection":
			XY = LambertAzimuthalEqualAreaProjection(XYZ); break;
		case "GnomonicProjection":
			XY = GnomonicProjection(XYZ); break;
		case "AzimuthalEquidistantDualProjection":
			XY = AzimuthalEquidistantDualProjection(XYZ); break;
		case "LambertAzimuthalEqualAreaDualProjection":
			XY = LambertAzimuthalEqualAreaDualProjection(XYZ); break;
		default:
			XY = AzimuthalEquidistantProjection(XYZ); break;
	}

	XY.X = Math.round(XY.X*10000)/10000;
	XY.Y = Math.round(XY.Y*10000)/10000;
	return XY;
}

//-------
// プロット方法各種
// 参考文献：
// 	正距円筒図法、正距方位図法、ランベルト正積方位図法、心射図法、ステレオ図法、モルワイデ図法、メルカトル図法
// 		Map Projections: A Working Manual 1987, Snyder, John P. http://pubs.er.usgs.gov/publication/pp1395
// 	ハンメル図法
// 		http://en.wikipedia.org/wiki/Hammer_projection
// 	メルカトル図法
// 		https://en.wikipedia.org/wiki/Mercator_projection
// 	ランベルト正積方位図法
// 		https://ja.wikipedia.org/wiki/%E3%83%A9%E3%83%B3%E3%83%99%E3%83%AB%E3%83%88%E6%AD%A3%E7%A9%8D%E6%96%B9%E4%BD%8D%E5%9B%B3%E6%B3%95
//-------
//正距円筒図法
function EquirectangularProjection(XYZ){
	var Adeg = RadToDeg(	Math.atan2(XYZ.Y,XYZ.X)	) + 180;
	if(Adeg >= 180){	Adeg -= 360;	}
	var Hdeg = RadToDeg(	Math.asin(XYZ.Z)	);

	var XY = {};
	XY.X = Adeg/180*cfg.chart.R*cfg.chart.X*2 + cfg.chart.center.X;
	XY.Y = Hdeg/180*cfg.chart.R*cfg.chart.X*2 + cfg.chart.center.Y

	XY.angle		= Adeg;
	XY.isVisible	= true;
	return XY;
}

//メルカトル図法
function MercatorProjection(XYZ){
	var Adeg = RadToDeg(	Math.atan2(XYZ.Y,XYZ.X)	) + 180;
	if(Adeg >= 180){	Adeg -= 360;	}
	var Hdeg = RadToDeg(	Math.asin(XYZ.Z)	);

	var XY = {};
	XY.X = Adeg/180*(2*cfg.chart.R)*cfg.chart.X + cfg.chart.center.X;

	if(89.9999 <= Hdeg){
		XY.Y =Math.log( Math.tan( Math.PI/4 + DegToRad(89.9999)/2 ) )/Math.PI*(2*cfg.chart.R)*cfg.chart.X + cfg.chart.center.Y;
	}else if(Hdeg <= -89.9999){
		XY.Y =Math.log( Math.tan( Math.PI/4 + DegToRad(-89.9999)/2 ) )/Math.PI*(2*cfg.chart.R)*cfg.chart.X + cfg.chart.center.Y;
	}else{
		XY.Y =Math.log( Math.tan( Math.PI/4 + DegToRad(Hdeg)/2 ) )/Math.PI*(2*cfg.chart.R)*cfg.chart.X + cfg.chart.center.Y;
	}

	XY.angle		= Adeg;
	XY.isVisible	= (Hdeg < boundaryCondition.MercatorProjection) ? true : false;
	return XY;
}

//モルワイデ図法
function MollweideProjection(XYZ){
	var Adeg = RadToDeg(	Math.atan2(XYZ.Y,XYZ.X)	) + 180;
	if(Adeg >= 180){	Adeg -= 360;	}
	var Hdeg = RadToDeg(	Math.asin(XYZ.Z)	);

	var M = 0;
	var F = Math.sin(M)+M-Math.PI*Math.sin(DegToRad(Hdeg));
	var dM = F/(Math.cos(M)+1);

	var count = 1;

	while(Math.abs(dM)>0.0001 && count < 15){
		count ++;
		M -= dM;
		F = Math.sin(M)+M-Math.PI*Math.sin(DegToRad(Hdeg));
		dM = F/(Math.cos(M)+1);
	}

	var XY = {};
	XY.X = DegToRad(Adeg)*Math.cos(M/2)/Math.PI*(2*cfg.chart.R)*cfg.chart.X + cfg.chart.center.X;
	XY.Y = Math.sin(M/2)*cfg.chart.R*cfg.chart.X + cfg.chart.center.Y;

	XY.angle		= Adeg;
	XY.isVisible	= true;
	return XY;
}

//ハンメル図法
function HammerProjection(XYZ){
	var Adeg = RadToDeg(	Math.atan2(XYZ.Y,XYZ.X)	) + 180;
	if(Adeg >= 180){	Adeg -= 360;	}
	var Hdeg = RadToDeg(	Math.asin(XYZ.Z)	);

	var XY = {};
	XY.X = 2*Math.cos(DegToRad(Hdeg))*Math.sin(DegToRad(Adeg)/2) / Math.sqrt(	1 + Math.cos(DegToRad(Hdeg))*Math.cos(DegToRad(Adeg)/2)	)*cfg.chart.R*cfg.chart.X + cfg.chart.center.X;
	XY.Y = Math.sin(DegToRad(Hdeg)) / Math.sqrt(	1 + Math.cos(DegToRad(Hdeg))*Math.cos(DegToRad(Adeg)/2)	)*cfg.chart.R*cfg.chart.X + cfg.chart.center.Y;

	XY.angle		= Adeg;
	XY.isVisible	= true;
	return XY;
}

//正距方位図法
function AzimuthalEquidistantProjection(XYZ){
	var Adeg = RadToDeg(	Math.atan2(XYZ.Y,XYZ.X)	) + 180;
	if(Adeg >= 180){	Adeg -= 360;	}
	var Hdeg = RadToDeg(	Math.asin(XYZ.Z)	);

	var XY = {};
	XY.X = (90-Hdeg)/90*Math.sin(DegToRad(Adeg))*cfg.chart.R*cfg.chart.X + cfg.chart.center.X;
	XY.Y = -(90-Hdeg)/90*Math.cos(DegToRad(Adeg))*cfg.chart.R*cfg.chart.X + cfg.chart.center.Y;

	XY.angle		= Hdeg;
	XY.isVisible	= (XYZ.Z>=boundaryCondition.AzimuthalEquidistantProjection) ? true : false;
	return XY;
}

//正距方位図法（二重円）
function AzimuthalEquidistantDualProjection(XYZ){
	var Adeg = RadToDeg(	Math.atan2(XYZ.Y,XYZ.X)	) + 180;
	if(Adeg >= 180){	Adeg -= 360;	}
	var Hdeg = RadToDeg(	Math.asin(XYZ.Z)	);

	var XY = {};
	if(XYZ.Z>=0){
		XY.X = (90-Hdeg)/90*Math.sin(DegToRad(Adeg))*cfg.chart.R*cfg.chart.X + cfg.chart.center.X - cfg.chart.R;
		XY.Y = -(90-Hdeg)/90*Math.cos(DegToRad(Adeg))*cfg.chart.R*cfg.chart.X + cfg.chart.center.Y;
	}else{
		XY.X = -(90+Hdeg)/90*Math.sin(DegToRad(Adeg))*cfg.chart.R*cfg.chart.X + cfg.chart.center.X + cfg.chart.R;
		XY.Y = -(90+Hdeg)/90*Math.cos(DegToRad(Adeg))*cfg.chart.R*cfg.chart.X + cfg.chart.center.Y;
	}

	XY.angle		= -RadToDeg(	Math.atan2(XYZ.Z,-XYZ.Y)	);
	XY.isVisible	= true;
	return XY;
}

//ランベルト正積方位図法
function LambertAzimuthalEqualAreaProjection(XYZ){
	var Adeg = RadToDeg(	Math.atan2(XYZ.Y,XYZ.X)	) + 180;
	if(Adeg >= 180){	Adeg -= 360;	}
	var Hdeg = RadToDeg(	Math.asin(XYZ.Z)	);

	var XY = {};
	XY.X = Math.sin(DegToRad(90-Hdeg)/2)*Math.sin(DegToRad(Adeg))*Math.sqrt(2)*cfg.chart.R*cfg.chart.X + cfg.chart.center.X;
	XY.Y = -Math.sin(DegToRad(90-Hdeg)/2)*Math.cos(DegToRad(Adeg))*Math.sqrt(2)*cfg.chart.R*cfg.chart.X + cfg.chart.center.Y;

	XY.angle		= 0;
	XY.isVisible	= (XYZ.Z>=boundaryCondition.LambertAzimuthalEqualAreaProjection) ? true : false;
	return XY;
}

//ランベルト正積方位図法（二重円）
function LambertAzimuthalEqualAreaDualProjection(XYZ){
	var Adeg = RadToDeg(	Math.atan2(XYZ.Y,XYZ.X)	) + 180;
	if(Adeg >= 180){	Adeg -= 360;	}
	var Hdeg = RadToDeg(	Math.asin(XYZ.Z)	);

	var XY = {};
	if(XYZ.Z>=0){
		XY.X = Math.sin(DegToRad(90-Hdeg)/2)*Math.sin(DegToRad(Adeg))*Math.sqrt(2)*cfg.chart.R*cfg.chart.X + cfg.chart.center.X - cfg.chart.R;
		XY.Y = -Math.sin(DegToRad(90-Hdeg)/2)*Math.cos(DegToRad(Adeg))*Math.sqrt(2)*cfg.chart.R*cfg.chart.X + cfg.chart.center.Y;
	}else{
		XY.X = -Math.sin(DegToRad(90+Hdeg)/2)*Math.sin(DegToRad(Adeg))*Math.sqrt(2)*cfg.chart.R*cfg.chart.X + cfg.chart.center.X + cfg.chart.R;
		XY.Y = -Math.sin(DegToRad(90+Hdeg)/2)*Math.cos(DegToRad(Adeg))*Math.sqrt(2)*cfg.chart.R*cfg.chart.X + cfg.chart.center.Y;
	}

	XY.angle		= -RadToDeg(	Math.atan2(XYZ.Z,-XYZ.Y)	);
	XY.isVisible	= true;
	return XY;
}

//ステレオ投影
function StereographicProjection(XYZ){
	var Adeg = RadToDeg(	Math.atan2(XYZ.Y,XYZ.X)	) + 180;
	if(Adeg >= 180){	Adeg -= 360;	}
	var Hdeg = RadToDeg(	Math.asin(XYZ.Z)	);

	var XY = {};
	if(XYZ.X != 1){
		XY.X = -XYZ.Y/(1-XYZ.X)*cfg.chart.R*cfg.chart.X + cfg.chart.center.X;
		XY.Y = XYZ.Z/(1-XYZ.X)*cfg.chart.R*cfg.chart.X + cfg.chart.center.Y;
	}else{
		XY.X = -XYZ.Y/(1-0.999)*cfg.chart.R*cfg.chart.X + cfg.chart.center.X;
		XY.Y = XYZ.Z/(1-0.999)*cfg.chart.R*cfg.chart.X + cfg.chart.center.Y;
	}

	XY.angle = 0;
	if(XYZ.X <= boundaryCondition.StereographicProjection){
		if( cfg.Sky.shape.indexOf("semi",0) >=0 ){
			XY.isVisible = (XYZ.Z >= 0) ? true: false;
		}else{
			XY.isVisible = true;
		}
	}else{
		XY.isVisible = false;
	}

	return XY;
}

//心射方位図法
function GnomonicProjection(XYZ){
	var Adeg = RadToDeg(	Math.atan2(XYZ.Y,XYZ.X)	) + 180;
	if(Adeg >= 180){	Adeg -= 360;	}
	var Hdeg = RadToDeg(	Math.asin(XYZ.Z)	);

	var XY = {};
	if(XYZ.X < 0){
		XY.X = XYZ.Y/XYZ.X*cfg.chart.R*cfg.chart.X + cfg.chart.center.X;
		XY.Y = -XYZ.Z/XYZ.X*cfg.chart.R*cfg.chart.X + cfg.chart.center.Y;
	}else if(XYZ.X == 0){
		XY.X = XYZ.Y/(-0.0001)*cfg.chart.R*cfg.chart.X + cfg.chart.center.X;
		XY.Y = -XYZ.Z/(-0.0001)*cfg.chart.R*cfg.chart.X + cfg.chart.center.Y;
	}else{
		XY.X = XYZ.Y/XYZ.X*cfg.chart.R*cfg.chart.X + cfg.chart.center.X;
		XY.Y = XYZ.Z/XYZ.X*cfg.chart.R*cfg.chart.X + cfg.chart.center.Y;
	}

	XY.angle		= 0;
	XY.isVisible	= (XYZ.X <= boundaryCondition.GnomonicProjection) ? true: false;
	return XY;
}

//----------
// 表示非表示の境界条件（発散や特異点）
//----------
var boundaryCondition = {
	MercatorProjection: 85.05113,
	AzimuthalEquidistantProjection: -0.05,
	LambertAzimuthalEqualAreaProjection: -0.65,
	StereographicProjection: 0.6,
	GnomonicProjection: -0.2
};
