//============================================================+
// File name   : modify_config.jsx
// Version     : 1.0.000
// Begin       : 2021-09-03
// Last Update : 2021-09-03
// Authors     : t.kenro - https://hoshifuru.jp - pr@hoshifuru.jp
// -------------------------------------------------------------------
// Copyright (C) 2021 t.kenro - hoshifuru.jp
//
// v1.0.0 20210903 modify_config.jsのv1.3.0を基に、加筆修正。setProjectionForcibly()を追加。setSaveAsAI()を追加。setRForcibly()を追加。setStar()から、cfg.Star.shapesを削除。setBtandInfo()を削除。

//星降る日時を設定する
function setStarryNight(YYYY,MM,DD,hh,mm,ss,isDST){
	if(typeof mm === "undefined") mm = 0;
	if(typeof ss === "undefined") ss = 0;
	if(typeof isDST === "undefined") isDST = false;
	cfg.When.YYYY = YYYY*1;
	cfg.When.MM = MM*1;
	cfg.When.DD = DD*1;
	cfg.When.hh = hh*1;
	cfg.When.mm = mm*1;
	cfg.When.ss = ss*1;
	cfg.When.isDST = isDST;
}

//星降る場所を設定する
function setStarryPlace(lon,lat,tZone,en,ja){
	cfg.Location.city = ja;
	cfg.Location.city_en = en;
	cfg.Location.Lon = lon;
	cfg.Location.Lat = lat;
	cfg.Location.timezone = tZone;
}

//表示・非表示を引数の配列をもとに設定する
function setOnOff(isDrawArray){
　	for(layer in isDrawArray){
		if( array_key_exists(layer,cfg.isDraw) ){
			cfg.isDraw[layer] = isDrawArray[layer];
		}
　	}
}

//星図の基本設定を行う
function setChart(pFormat,pOrientation,type,Adeg,Hdeg,Rdeg){
	if(typeof Adeg === "undefined") Adeg = 0;
	if(typeof Hdeg === "undefined") Hdeg = 0;
	if(typeof Rdeg === "undefined") Rdeg = 0;

	setPaper(pFormat,pOrientation);

	switch(type){
	//簡易設定
		case "Sky":
		case "SKY":
		case "空全体":
		case "SkyS":
		case "SKYS":
		case "空全体南":
			setProjection("正距方位");
			setLookAt("地平座標",180+Adeg,Hdeg,Rdeg);
			setSkyShape("円");
			break;
		case "SkyN":
		case "SKYN":
		case "空全体北":
			setProjection("正距方位");
			setLookAt("地平座標",0+Adeg,Hdeg,Rdeg);
			setSkyShape("円");
			break;
		case "N":
		case "北":
			setProjection("ステレオ");
			setLookAt("地平座標",0+Adeg,Hdeg,Rdeg);
			setSkyShape("半円");
			break;
		case "NE":
		case "北東":
			setProjection("ステレオ");
			setLookAt("地平座標",45+Adeg,Hdeg,Rdeg);
			setSkyShape("半円");
			break;
		case "E":
		case "東":
			setProjection("ステレオ");
			setLookAt("地平座標",90+Adeg,Hdeg,Rdeg);
			setSkyShape("半円");
			break;
		case "SE":
		case "南東":
			setProjection("ステレオ");
			setLookAt("地平座標",135+Adeg,Hdeg,Rdeg);
			setSkyShape("半円");
			break;
		case "S":
		case "南":
			setProjection("ステレオ");
			setLookAt("地平座標",180+Adeg,Hdeg,Rdeg);
			setSkyShape("半円");
			break;
		case "SW":
		case "南西":
			setProjection("ステレオ");
			setLookAt("地平座標",225+Adeg,Hdeg,Rdeg);
			setSkyShape("半円");
			break;
		case "W":
		case "西":
			setProjection("ステレオ");
			setLookAt("地平座標",270+Adeg,Hdeg,Rdeg);
			setSkyShape("半円");
			break;
		case "NW":
		case "北西":
			setProjection("ステレオ");
			setLookAt("地平座標",315+Adeg,Hdeg,Rdeg);
			setSkyShape("半円");
			break;
	//パラメータ指定
		case "Zenith":
		case "Z":
		case "HorizontalZ":
		case "HZ":
		case "H":
		case "天頂":
			setProjection("正距方位");
			setLookAt("地平座標",180+Adeg,Hdeg,Rdeg);
			setSkyShape("円");
			break;
		case "NESW":
		case "HorizontalNESW":
		case "HNESW":
		case "東西南北":
			setProjection("ステレオ");
			setLookAt("地平座標",180+Adeg,Hdeg,Rdeg);
			setSkyShape("半円");
			break;
		case "Equatorial":
		case "Eq":
		case "赤道":
		case "VernalEquinox":
		case "春分点":
			setProjection("正距円筒");
			setLookAt("赤道座標",Adeg,Hdeg,Rdeg);
			setSkyShape("矩形");
			break;
		case "AutumnalEquinox":
		case "秋分点":
			setProjection("正距円筒");
			setLookAt("赤道座標",180+Adeg,Hdeg,Rdeg);
			setSkyShape("矩形");
			break;
		case "Galactic":
		case "G":
		case "銀河":
			setProjection("ハンメル");
			setLookAt("銀河座標",Adeg,Hdeg,Rdeg);
			setSkyShape("楕円");
			break;
		case "Ecliptic":
		case "Ec":
		case "黄道":
			setProjection("ハンメル");
			setLookAt("黄道座標",Adeg,Hdeg,Rdeg);
			setSkyShape("楕円");
			break;
		default:
			setProjection("正距方位");
			setLookAt("地平座標",180+Adeg,Hdeg,Rdeg);
			setSkyShape("円");
			break;
	}
	setChartDetail();
}

//星図の用紙サイズを設定する
function setPaper(format,orientation){
	var pFormat,pOrientation;
	var pFormat = ( array_key_exists(format,paperWH) )? format : "A4";

	var o = orientation;
	if(o == "P"||o == "Portrait"||o == "縦"){			pOrientation = "P";
	}else if(o == "L"||o == "Landscape"||o == "横"){		pOrientation = "L";
	}else{												pOrientation = "P";	}

	cfg.paper.format = pFormat;
	cfg.paper.orientation = pOrientation;
	cfg.paper.w = (pOrientation == "P")? paperWH[pFormat]["w"] :paperWH[pFormat]["h"];
	cfg.paper.h = (pOrientation == "P")? paperWH[pFormat]["h"] :paperWH[pFormat]["w"];
}

//半径1の球から紙面に描画する時の描画方法を設定する
function setProjection(type){
	var projection;
	switch(type){
		case "Equirectangular":
		case "正距円筒":
			projection = "EquirectangularProjection";
			break;
		case "Mercator":
		case "メルカトル":
			projection = "MercatorProjection";
			break;
		case "Mollweide":
		case "モルワイデ":
			projection = "MollweideProjection";
			break;
		case "Hammer":
		case "ハンメル":
			projection = "HammerProjection";
			break;
		case "AzimuthalEquidistant":
		case "正距方位":
			projection = "AzimuthalEquidistantProjection";
			break;
		case "AzimuthalEquidistantDual":
		case "正距方位二重円":
			projection = "AzimuthalEquidistantDualProjection";
			break;
		case "LambertAzimuthalEqualArea":
		case "ランベルト正積方位":
			projection = "LambertAzimuthalEqualAreaProjection";
			break;
		case "LambertAzimuthalEqualAreaDual":
		case "ランベルト正積方位二重円":
			projection = "LambertAzimuthalEqualAreaDualProjection";
			break;
		case "Gnomonic":
		case "心射方位":
			projection = "GnomonicProjection";
			break;
		case "Stereographic":
		case "ステレオ":
			projection = "StereographicProjection";
			break;
		default:
			projection = "AzimuthalEquidistantProjection";
			break;
	}
	cfg.chart.projection = projection;
}

//描画図法を強制的に変更する
function setProjectionForcibly(projection){
	if(typeof projection !== "undefined"){
		if(projection != ""){
			setProjection(projection);

			switch(projection){
				case "Equirectangular":
				case "Mercator":
				case "Gnomonic":
					setSkyShape("矩形");
					break;
				case "Mollweide":
				case "Hammer":
					setSkyShape("楕円");
					break;
				case "AzimuthalEquidistant":
				case "LambertAzimuthalEqualArea":
					setSkyShape("円");
					break;
				case "Stereographic":
					setSkyShape("正方形");
					break;
			}
		}
	}
}

//視線方向（星図の中心）を設定する
function setLookAt(coordinate,Adeg,Hdeg,Rdeg){
	if(typeof Hdeg === "undefined") Hdeg = 0;
	if(typeof Rdeg === "undefined") Rdeg = 0;

	var addRZ,to;
	switch(coordinate){
		case "Horizontal":
		case "地平座標":
			addRZ = -Adeg;
			to = "Horizontal";
			break;
		case "Equatorial":
		case "赤道座標":
			addRZ = -(180 - Adeg);
			to = "Equatorial";
			break;
		case "Galactic":
		case "銀河座標":
			addRZ = -(180 - Adeg);
			to = "Galactic";
			break;
		case "Ecliptic":
		case "黄道座標":
			addRZ = -(180 - Adeg);
			to = "Ecliptic";
			break;
		default:
			addRZ = -Adeg;
			to = "Horizontal";
			break;
	}
	cfg.chart.to = to;
	cfg.chart.lookAtA = Adeg;
	cfg.chart.lookAtH = Hdeg;
	cfg.chart.lookAtR = Rdeg;

	addR.Z = addRZ;
	addR.Y = -Hdeg;
	addR.X = Rdeg;
}

//空領域の形状を設定する
function setSkyShape(shape){
	var selected;
	switch(shape){
		case "circle":
		case "円":
			selected = "circle";
			break;
		case "semi_circle":
		case "半円":
			selected = "semi_circle";
			break;
		case "rectangular":
		case "矩形":
		case "長方形":
			selected = "rectangular";
			break;
		case "semi_rectangular":
		case "半矩形":
		case "半長方形":
			selected = "semi_rectangular";
			break;
		case "ellipse":
		case "楕円":
			selected = "ellipse";
			break;
		case "semi_ellipse":
		case "半楕円":
			selected = "semi_ellipse";
			break;
		case "square":
		case "正方形":
		case "四角":
			selected = "square";
			break;
		case "semi_square":
		case "半正方形":
		case "半四角":
			selected = "semi_square";
			break;
		case "dualcircle":
		case "二重円":
			selected = "dualcircle";
			break;
		default:
			selected = "circle";
			break;
	}
	cfg.Sky.shape = selected;
}

//星図の詳細（星図中心・半径、各種情報の表示位置）を設定
function setChartDetail(){
	setChartPosition();
	setInfoPosition();
}

//星図の中心位置・半径を設定する
function setChartPosition(){
	var p = cfg.paper;
	var to = cfg.chart.to;
	var plot = cfg.chart.projection;

	var cX = p["w"]/2;
	var cY = p["h"]/2;
	var R = (p["w"]-p["margin"])/4;

	if(p["orientation"] == "L"){
		if(to == "Horizontal"){
			if(plot == "StereographicProjection"){
				R = (p["w"]-p["margin"]*2)/2;
				cY -= R/2;
			}else if(plot == "AzimuthalEquidistantProjection" || plot == "LambertAzimuthalEqualAreaProjection"){
				R = (p["h"]-p["margin"]*2)/2;
			}
		}
	}
	if(p["orientation"] == "P"){
		if(to == "Horizontal"){
			R = (p["w"]-p["margin"]*2)/2;
			if(plot == "StereographicProjection"){
				cY += R/2*0.2;
			}
		}else{
			cY += R*1.2;
		}
	}

	setChartC(cX,cY,R);
	setSkyC(cX,cY,R);
}

//星図の縮尺を設定する
function setX(num){
	cfg.chart.X	= num;
}

//星図の中心位置・半径を設定する
function setChartC(cX,cY,R){
	if(typeof R === "undefined") R = "";
	cfg.chart.center.X	= cX;
	cfg.chart.center.Y	= cY;
	if(R != ""){
		cfg.chart.R	= R;
	}
}

//空の中心位置・半径を設定する
function setSkyC(cX,cY,R){
	if(typeof R === "undefined") R = "";
	cfg.Sky.cX	= cX;
	cfg.Sky.cY	= cY;
	if(R != ""){
		cfg.Sky.R	= R;
	}
}

//星図の半径・空の半径を強制的に設定する
function setRForcibly(R){
	if(typeof R === "undefined") R = "";
	if(R != ""){	cfg.chart.R	= R;	}
	if(R != ""){	cfg.Sky.R	= R;	}
}

//日時・場所・等級凡例の配置を設定する
function setInfoPosition(){
	var p = cfg.paper;
	var to = cfg.chart.to;
	var plot = cfg.chart.projection;

	setINFOdate("C","T");
	setINFOplace("C","T");
	setMagExsample("h","C","B");

	if(p["orientation"] == "L"){
		if(to == "Horizontal"){
			if(plot == "AzimuthalEquidistantProjection" || plot == "LambertAzimuthalEqualAreaProjection"){
				setINFOdate("L","T");
				setINFOplace("L","T");
				setMagExsample("v","R","B");
			}
		}
	}
	if(p["orientation"] == "P"){
		if(to == "Horizontal"){
			if(plot == "StereographicProjection"){
				setMagExsample("h","C","M");
			}
		}else{
			if(plot != "AzimuthalEquidistantProjection" || plot != "LambertAzimuthalEqualAreaProjection"){
				setMagExsample("h","C","M");
			}
		}
	}
}

//等級凡例の配置を設定
function setMagExsample(type,align,valign){
	cfg.MagEx.type		= type;
	cfg.MagEx.align		= align;
	cfg.MagEx.valign	= valign;
}

//日時の配置を設定
function setINFOdate(align,valign){
	cfg.Info.d_align	= align;
	cfg.Info.d_valign	= valign;
}

//場所の配置を設定
function setINFOplace(align,valign){
	cfg.Info.p_align	= align;
	cfg.Info.p_valign	= valign;
}

//天の川の設定
function setMilkyWay(type,dotShape){
	if(typeof dotShape === "undefined") dotShape = "";
	cfg.MilkyWay.type = type;
	cfg.MilkyWay.width = (type == "line")? 1.5 : 0.1;
	if(dotShape != ""){	cfg.MilkyWay.shape	= dotShape;	}
}

//星の各種設定
function setStar(shape,maxMag,minMag,M1R,endStr,Rratio){
	if(typeof shape === "undefined") shape = "";
	if(typeof maxMag === "undefined") maxMag = "";
	if(typeof minMag === "undefined") minMag = "";
	if(typeof M1R === "undefined") M1R = "";
	if(typeof endStr === "undefined") endStr = "";
	if(typeof Rratio === "undefined") Rratio = "";
	if(shape != ""){	cfg.Star.shape	= shape;	}
	if(maxMag != ""){	cfg.Star.maxMag	= maxMag*1;	}
	if(minMag != ""){	cfg.Star.minMag	= minMag*1;	}
	if(M1R != ""){		cfg.Star.M1R	= M1R*1;	}
	if(endStr != ""){	cfg.Star.endString	= endStr;	}
	if(Rratio != ""){	cfg.Star.Rratio	= Rratio*1;	}
}

//星座名の設定
function setStarName(TorF,priority){
	if(typeof priority === "undefined") priority = "";
	cfg.Star.name.isDraw = TorF;
	if(priority != ""){	cfg.Star.name.priority	= priority;	}
}

//流星群の設定
function setMeteor(id){
	if(typeof id === "undefined") id = "";
	if(id != ""){	cfg.Meteor.id	= id;	}
}

//スカイラインの設定
function setLandscape(skyline,type){
	if(typeof skyline === "undefined") skyline = "";
	if(typeof type === "undefined") type = "";
	if(skyline != ""){	cfg.Landscape.skyline	= skyline;	}
	if(type != ""){	cfg.Landscape.type	= type;	}
}

//太陽・惑星・月の設定
function setSunMoon(sShape,pShape){
	if(typeof sShape === "undefined") sShape = "";
	if(typeof pShape === "undefined") pShape = "";
	if(sShape != ""){	cfg.SunMoon.sShape	= sShape;	}
	if(pShape != ""){	cfg.SunMoon.pShape	= pShape;	}
}

//彗星の設定
function setComet(cometID){
	if(typeof cometID === "undefined") cometID = "";
	if(cometID != ""){	cfg.Comet.id	= cometID;	}
}

//グリッド情報を設定
function setGrid(coordinate,Astep,Hstep){
	if(typeof Astep === "undefined") Astep = "";
	if(typeof Hstep === "undefined") Hstep = "";
	var type;
	switch(coordinate){
		case "Horizontal":
		case "地平座標":
		case "方位高度":
			type = "Horizontal";
			break;
		case "Equatorial":
		case "赤道座標":
			type = "Equatorial";
			break;
		case "Galactic":
		case "銀河座標":
			type = "Galactic";
			break;
		case "Ecliptic":
		case "黄道座標":
			type = "Ecliptic";
			break;
		default:
			type = "Horizontal";
			break;
	}
	cfg.Grid.datatype = type;
	if(Astep != ""){	cfg.Grid.degStepA = Astep;	}
	if(Hstep != ""){	cfg.Grid.degStepH = Hstep;	}
}

//言語を設定
function setLang(lang){
	//方位表示、場所表示、コピーライト表示は、原則英語
	cfg.Star.name.lang = lang;
	cfg.ConstName.lang = lang;
//	cfg.NESW.lang = lang;
	cfg.SunMoon.lang = lang;
	cfg.Comet.lang = lang;
	cfg.Meteor.lang = lang;
	cfg.Info.lang = lang;
//	cfg.Info.date.lang = lang;
//	cfg.Info.title.lang = lang;
//	cfg.Copyright.lang = lang;
}

//基本の線種を設定
function setLine(type){
//	cfg.ConstLine.alpha = lineSet[type]["alpha"];
	cfg.ConstLine.width = lineSet[type]["width"];
	cfg.ConstLine.dash = lineSet[type]["dash"];
//	cfg.ConstLine.cap = lineSet[type]["cap"];
//	cfg.ConstLine.join = lineSet[type]["join"];
}

//基本の色味を設定
function setColor(color){
	cfg.chart.color = color;
	cCfg	= colorSet[color];

	cfg.Star.color = cCfg["star"];
	cfg.Star.lineColor = cCfg["star"];
	cfg.Star.name.fColor = cCfg["starname"];
	cfg.ConstName.fColor = cCfg["name"];
	cfg.ConstLine.color = cCfg["line"];
	cfg.Landscape.color = cCfg["landscape"];
	cfg.Landscape.lineColor = cCfg["landscape"];
	cfg.Sky.color = cCfg["skyColor"];
//	cfg.Sky.isGradation = cCfg["skyisGradation"];
//	cfg.Sky.g0Color = cCfg["skyG0Color"];
//	cfg.Sky.g1Color = cCfg["skyG1Color"];
	cfg.Sky.border.color = cCfg["borderColor"];
	cfg.BG.color = cCfg["BGColor"];
	cfg.MilkyWay.color = cCfg["milkyway"];
	cfg.SunMoon.pColor = cCfg["planet"];
	cfg.SunMoon.fColor = cCfg["planetf"];
	cfg.SunMoon.sColor = cCfg["sunColor"];
	cfg.SunMoon.sLineColor = cCfg["sunLineColor"];
	cfg.SunMoon.mColor = cCfg["moonColor"];
	cfg.SunMoon.mColorBase = cCfg["moonBaseColor"];
//	cfg.info.title.fColor = cCfg["title"];
	cfg.Info.fColor = cCfg["place"];
//	cfg.info.date.fColor = cCfg["title"];
//	cfg.info.place.fColor = cCfg["place"];
	cfg.MagEx.fColor = cCfg["title"];
//	cfg.Copyright.fColor = cCfg["BGColor"];
	cfg.NESW.fColor = cCfg["NESWfColor"];
	cfg.NESW.BGColor = cCfg["NESWBGColor"];
	cfg.Grid.color = cCfg["grid"];
	cfg.Grid.lineColor = cCfg["grid"];
	cfg.Grid.degText.fColor = cCfg["gridfColor"];
//	cfg.Story.fColor = cCfg["place"];
	cfg.Meteor.color = cCfg["meteorColor"];
	cfg.Meteor.fColor = cCfg["meteorColor"];
	cfg.Comet.pColor = cCfg["cometColor"];
	cfg.Comet.fColor = cCfg["cometColor"];
	cfg.Comet.orbit.color = cCfg["cometLineColor"];
	cfg.Comet.orbit.fColor = cCfg["starname"];
	cfg.Comet.orbit.sColor = cCfg["cometDateColor"];
//	cfg.Copyright.fColor = cCfg["copyright"];
}

//星座名・星座線の表示設定（全表示、黄道１２宮のみなど）
function setConstellation(type){
	if(typeof type === "undefined") type = "all";
	var sType = ( array_key_exists(type,constSet) ) ? type : "all";

	cfg.ConstLine.isSimplify	= (sType == "all") ? false : true;
	cfg.ConstName.isSimplify	= (sType == "all") ? false : true;

	if(type != "all"){
		cfg.ConstLine.list	= constSet[sType];
		cfg.ConstName.list	= constSet[sType];
	}
}

//大気による浮き上がり効果（大気差補正）を考慮するかどうか？
function setIsCorrectH(OnOff){
	cfg.isCorrectH = OnOff;
}

//描画後にAIを保存するかどうか
function setSaveAsAI(OnOff){
	cfg.isSaveAsAI = OnOff;
}

//-----------------------------------
// setup starchart condition just before drawing on pdf
//-----------------------------------
function setupStarChart(){
	var d = cfg.When;
	//地方恒星時を計算
	var LST = calkLSThr(d["YYYY"],d["MM"],d["DD"],d["hh"],d["mm"],d["ss"],d["isDST"],cfg.Location.timezone,cfg.Location.Lon);
	setLSThr(LST);

	//月齢を計算
	var mAge = calcMoonAge(d["YYYY"],d["MM"],d["DD"],d["hh"],d["mm"],d["ss"],d["isDST"],cfg.Location.timezone);
	setMoonAge(mAge);

	//空の描画境界値を計算
	boundaryCondition.AzimuthalEquidistantProjection		= Math.cos(Math.PI/2/cfg.chart.X);
	boundaryCondition.LambertAzimuthalEqualAreaProjection	= 1-1/Math.pow(cfg.chart.X,2);
	boundaryCondition.StereographicProjection				= ( cfg.Sky.shape.indexOf("circle") >= 0 ) ? -(Math.pow(cfg.chart.X,2)-1)/(Math.pow(cfg.chart.X,2)+1) : 0.8;	//0.8は、矩形の空の形を考慮した値。空が円で良ければ0.6となる。
	boundaryCondition.GnomonicProjection					= -Math.sqrt( Math.pow(cfg.chart.X,2) / (Math.pow(cfg.chart.X,2)　+　Math.pow(1.55,2)) );	//1.55は、空の形が円であれば1で良い。ここでは矩形の空の形を考慮し、1.55とした。
	//風景（couple,chaildrenの正規化）
	adjustLandscape();
}

//風景（恋人・子供）の場合、中央に人物が来るように、風景開始位置を調節
function adjustLandscape(){
	var skyline = cfg.Landscape.skyline;
	var to = cfg.chart.to;

	cfg.Landscape.angle = 0;

	if(skyline == "couple" || skyline == "children"){
		if(to == "Horizontal"){
			cfg.Landscape.angle = cfg.chart.lookAtA + 180;
		}
	}
}

//============
// 追加関数
//============
function array_key_exists(keyName,ar){
	var is = false;
	for(key in ar){
		if(keyName == key){	is = true;	break;	}
	}
    return is;
}